@@ -38,9 +38,7 @@
 #include "catalog/namespace.h"
 #include "mb/pg_wchar.h"
 #include "utils/builtins.h"
-#include "utils/memdebug.h"
 #include "utils/memutils.h"
-#include "utils/relcache.h"
 #include "utils/syscache.h"
 #include "varatt.h"
 
@@ -99,13 +97,6 @@ static char *perform_default_encoding_conversion(const char *src,
 												 int len, bool is_client_to_server);
 static int	cliplen(const char *str, int len, int limit);
 
-pg_attribute_noreturn()
-static void report_invalid_encoding_int(int encoding, const char *mbstr,
-										int mblen, int len);
-
-pg_attribute_noreturn()
-static void report_invalid_encoding_db(const char *mbstr, int mblen, int len);
-
 
 /*
  * Prepare for a future call to SetClientEncoding.  Success should mean
@@ -320,7 +311,7 @@ InitializeClientEncoding(void)
 	{
 		Oid			utf8_to_server_proc;
 
-		AssertCouldGetRelation();
+		Assert(IsTransactionState());
 		utf8_to_server_proc =
 			FindDefaultConversionProc(PG_UTF8,
 									  current_server_encoding);
@@ -1028,128 +1019,11 @@ pg_encoding_wchar2mb_with_len(int encoding,
 	return pg_wchar_table[encoding].wchar2mb_with_len(from, (unsigned char *) to, len);
 }
 
-/*
- * Returns the byte length of a multibyte character sequence in a
- * null-terminated string.  Raises an illegal byte sequence error if the
- * sequence would hit a null terminator.
- *
- * The caller is expected to have checked for a terminator at *mbstr == 0
- * before calling, but some callers want 1 in that case, so this function
- * continues that tradition.
- *
- * This must only be used for strings that have a null-terminator to enable
- * bounds detection.
- */
-int
-pg_mblen_cstr(const char *mbstr)
-{
-	int			length = pg_wchar_table[DatabaseEncoding->encoding].mblen((const unsigned char *) mbstr);
-
-	/*
-	 * The .mblen functions return 1 when given a pointer to a terminator.
-	 * Some callers depend on that, so we tolerate it for now.  Well-behaved
-	 * callers check the leading byte for a terminator *before* calling.
-	 */
-	for (int i = 1; i < length; ++i)
-		if (unlikely(mbstr[i] == 0))
-			report_invalid_encoding_db(mbstr, length, i);
-
-	/*
-	 * String should be NUL-terminated, but checking that would make typical
-	 * callers O(N^2), tripling Valgrind check-world time.  Unless
-	 * VALGRIND_EXPENSIVE, check 1 byte after each actual character.  (If we
-	 * found a character, not a terminator, the next byte must be a terminator
-	 * or the start of the next character.)  If the caller iterates the whole
-	 * string, the last call will diagnose a missing terminator.
-	 */
-	if (mbstr[0] != '\0')
-	{
-#ifdef VALGRIND_EXPENSIVE
-		VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, strlen(mbstr));
-#else
-		VALGRIND_CHECK_MEM_IS_DEFINED(mbstr + length, 1);
-#endif
-	}
-
-	return length;
-}
-
-/*
- * Returns the byte length of a multibyte character sequence bounded by a range
- * [mbstr, end) of at least one byte in size.  Raises an illegal byte sequence
- * error if the sequence would exceed the range.
- */
-int
-pg_mblen_range(const char *mbstr, const char *end)
-{
-	int			length = pg_wchar_table[DatabaseEncoding->encoding].mblen((const unsigned char *) mbstr);
-
-	Assert(end > mbstr);
-
-	if (unlikely(mbstr + length > end))
-		report_invalid_encoding_db(mbstr, length, end - mbstr);
-
-#ifdef VALGRIND_EXPENSIVE
-	VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, end - mbstr);
-#else
-	VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, length);
-#endif
-
-	return length;
-}
-
-/*
- * Returns the byte length of a multibyte character sequence bounded by a range
- * extending for 'limit' bytes, which must be at least one.  Raises an illegal
- * byte sequence error if the sequence would exceed the range.
- */
-int
-pg_mblen_with_len(const char *mbstr, int limit)
-{
-	int			length = pg_wchar_table[DatabaseEncoding->encoding].mblen((const unsigned char *) mbstr);
-
-	Assert(limit >= 1);
-
-	if (unlikely(length > limit))
-		report_invalid_encoding_db(mbstr, length, limit);
-
-#ifdef VALGRIND_EXPENSIVE
-	VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, limit);
-#else
-	VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, length);
-#endif
-
-	return length;
-}
-
-
-/*
- * Returns the length of a multibyte character sequence, without any
- * validation of bounds.
- *
- * PLEASE NOTE:  This function can only be used safely if the caller has
- * already verified the input string, since otherwise there is a risk of
- * overrunning the buffer if the string is invalid.  A prior call to a
- * pg_mbstrlen* function suffices.
- */
-int
-pg_mblen_unbounded(const char *mbstr)
-{
-	int			length = pg_wchar_table[DatabaseEncoding->encoding].mblen((const unsigned char *) mbstr);
-
-	VALGRIND_CHECK_MEM_IS_DEFINED(mbstr, length);
-
-	return length;
-}
-
-/*
- * Historical name for pg_mblen_unbounded().  Should not be used and will be
- * removed in a later version.
- */
+/* returns the byte length of a multibyte character */
 int
 pg_mblen(const char *mbstr)
 {
-	return pg_mblen_unbounded(mbstr);
+	return pg_wchar_table[DatabaseEncoding->encoding].mblen((const unsigned char *) mbstr);
 }
 
 /* returns the display length of a multibyte character */
@@ -1171,14 +1045,14 @@ pg_mbstrlen(const char *mbstr)
 
 	while (*mbstr)
 	{
-		mbstr += pg_mblen_cstr(mbstr);
+		mbstr += pg_mblen(mbstr);
 		len++;
 	}
 	return len;
 }
 
 /* returns the length (counted in wchars) of a multibyte string
- * (stops at the first of "limit" or a NUL)
+ * (not necessarily NULL terminated)
  */
 int
 pg_mbstrlen_with_len(const char *mbstr, int limit)
@@ -1191,7 +1065,7 @@ pg_mbstrlen_with_len(const char *mbstr, int limit)
 
 	while (limit > 0 && *mbstr)
 	{
-		int			l = pg_mblen_with_len(mbstr, limit);
+		int			l = pg_mblen(mbstr);
 
 		limit -= l;
 		mbstr += l;
@@ -1214,7 +1088,7 @@ pg_mbcliplen(const char *mbstr, int len, int limit)
 }
 
 /*
- * pg_mbcliplen with specified encoding; string must be valid in encoding
+ * pg_mbcliplen with specified encoding
  */
 int
 pg_encoding_mbcliplen(int encoding, const char *mbstr,
@@ -1261,7 +1135,7 @@ pg_mbcharcliplen(const char *mbstr, int len, int limit)
 
 	while (len > 0 && *mbstr)
 	{
-		l = pg_mblen_with_len(mbstr, len);
+		l = pg_mblen(mbstr);
 		nch++;
 		if (nch > limit)
 			break;
@@ -1825,25 +1699,18 @@ check_encoding_conversion_args(int src_encoding,
  * report_invalid_encoding: complain about invalid multibyte character
  *
  * note: len is remaining length of string, not length of character;
- * len must be greater than zero (or we'd neglect initializing "buf").
+ * len must be greater than zero, as we always examine the first byte.
  */
 void
 report_invalid_encoding(int encoding, const char *mbstr, int len)
 {
-	int			l = pg_encoding_mblen_or_incomplete(encoding, mbstr, len);
-
-	report_invalid_encoding_int(encoding, mbstr, l, len);
-}
-
-static void
-report_invalid_encoding_int(int encoding, const char *mbstr, int mblen, int len)
-{
+	int			l = pg_encoding_mblen(encoding, mbstr);
 	char		buf[8 * 5 + 1];
 	char	   *p = buf;
 	int			j,
 				jlimit;
 
-	jlimit = Min(mblen, len);
+	jlimit = Min(l, len);
 	jlimit = Min(jlimit, 8);	/* prevent buffer overrun */
 
 	for (j = 0; j < jlimit; j++)
@@ -1860,36 +1727,22 @@ report_invalid_encoding_int(int encoding, const char *mbstr, int mblen, int len)
 					buf)));
 }
 
-static void
-report_invalid_encoding_db(const char *mbstr, int mblen, int len)
-{
-	report_invalid_encoding_int(GetDatabaseEncoding(), mbstr, mblen, len);
-}
-
 /*
  * report_untranslatable_char: complain about untranslatable character
  *
  * note: len is remaining length of string, not length of character;
- * len must be greater than zero (or we'd neglect initializing "buf").
+ * len must be greater than zero, as we always examine the first byte.
  */
 void
 report_untranslatable_char(int src_encoding, int dest_encoding,
 						   const char *mbstr, int len)
 {
-	int			l;
+	int			l = pg_encoding_mblen(src_encoding, mbstr);
 	char		buf[8 * 5 + 1];
 	char	   *p = buf;
 	int			j,
 				jlimit;
 
-	/*
-	 * We probably could use plain pg_encoding_mblen(), because
-	 * gb18030_to_utf8() verifies before it converts.  All conversions should.
-	 * For src_encoding!=GB18030, len>0 meets pg_encoding_mblen() needs.  Even
-	 * so, be defensive, since a buggy conversion might pass invalid data.
-	 * This is not a performance-critical path.
-	 */
-	l = pg_encoding_mblen_or_incomplete(src_encoding, mbstr, len);
 	jlimit = Min(l, len);
 	jlimit = Min(jlimit, 8);	/* prevent buffer overrun */
 
