@@ -26,18 +26,32 @@ INSERT INTO OID_TBL(f1) VALUES (' - 500');
 INSERT INTO OID_TBL(f1) VALUES ('32958209582039852935');
 INSERT INTO OID_TBL(f1) VALUES ('-23582358720398502385');
 
-SELECT '' AS six, * FROM OID_TBL;
+SELECT * FROM OID_TBL;
 
-SELECT '' AS one, o.* FROM OID_TBL o WHERE o.f1 = 1234;
+-- Also try it with non-error-throwing API
+SELECT pg_input_is_valid('1234', 'oid');
+SELECT pg_input_is_valid('01XYZ', 'oid');
+SELECT * FROM pg_input_error_info('01XYZ', 'oid');
+SELECT pg_input_is_valid('9999999999', 'oid');
+SELECT * FROM pg_input_error_info('9999999999', 'oid');
 
-SELECT '' AS five, o.* FROM OID_TBL o WHERE o.f1 <> '1234';
+-- While we're here, check oidvector as well
+SELECT pg_input_is_valid(' 1 2  4 ', 'oidvector');
+SELECT pg_input_is_valid('01 01XYZ', 'oidvector');
+SELECT * FROM pg_input_error_info('01 01XYZ', 'oidvector');
+SELECT pg_input_is_valid('01 9999999999', 'oidvector');
+SELECT * FROM pg_input_error_info('01 9999999999', 'oidvector');
 
-SELECT '' AS three, o.* FROM OID_TBL o WHERE o.f1 <= '1234';
+SELECT o.* FROM OID_TBL o WHERE o.f1 = 1234;
 
-SELECT '' AS two, o.* FROM OID_TBL o WHERE o.f1 < '1234';
+SELECT o.* FROM OID_TBL o WHERE o.f1 <> '1234';
 
-SELECT '' AS four, o.* FROM OID_TBL o WHERE o.f1 >= '1234';
+SELECT o.* FROM OID_TBL o WHERE o.f1 <= '1234';
 
-SELECT '' AS three, o.* FROM OID_TBL o WHERE o.f1 > '1234';
+SELECT o.* FROM OID_TBL o WHERE o.f1 < '1234';
+
+SELECT o.* FROM OID_TBL o WHERE o.f1 >= '1234';
+
+SELECT o.* FROM OID_TBL o WHERE o.f1 > '1234';
 
 DROP TABLE OID_TBL;
